@@ -2,47 +2,55 @@
 #' @title Compute linear impulse responses
 #' @description Compute linear impulse responses with local projections by Jordà (2005).
 #'
-#' @param data_set_df A \link{data.frame}() containing all endogenous variables for the VAR. The column order
+#' @param endog_data A \link{data.frame}, containing the endogenous variables for the VAR. The column order
 #'                    is used for the Cholesky decomposition.
 #' @param lags_criterion NaN or character. NaN means that the number of lags
-#'         will be given at \emph{lags_lin}. The character refers to the corresponding lag length criterion ('AICc', 'AIC' or 'BIC').
-#' @param lags_lin NaN or integer. NaN if lag length criterion is used. Integer for number of lags for linear VAR.
-#' @param max_lags NaN or integer. Maximum number of lags if \emph{lags_criterion} is character with lag length criterion. NaN otherwise.
+#'         will be given at \emph{lags_endog_lin}. The character specifies the lag length criterion ('AICc', 'AIC' or 'BIC').
+#' @param lags_endog_lin NaN or integer. NaN if lag length criterion is used. Integer for number of lags for \emph{endog_data}.
+#' @param max_lags NaN or integer. Maximum number of lags if \emph{lags_criterion} is given. NaN otherwise.
 #' @param trend Integer. No trend =  0 , include trend = 1, include trend and quadratic trend = 2.
 #' @param shock_type Integer. Standard deviation shock = 0, unit shock = 1.
 #' @param confint Double. Width of confidence bands. 68\% = 1, 90\% = 1.65, 95\% = 1.96.
 #' @param hor Integer. Number of horizons for impulse responses.
+#' @param exog_data A \link{data.frame}, containing exogenous variables for the VAR. The row length has to be the same as \emph{endog_data}.
+#'                 Lag lengths for exogenous variables have to be given and will no be determined via a lag length criterion.
+#' @param lags_exog Integer. Number of lags for the exogenous variables.
+#' @param contemp_data A \link{data.frame}, containing exogenous data with contemporaneous impact. The row length has to be the same as \emph{endog_data}.
+#' @param num_cores NULL or Integer. The number of cores to use for the estimation. If NULL, the function will
+#'                 use the maximum number of cores minus one.
 #'
-#' @return A list with impulse responses and their robust confidence bands.
-#' It also returns a list named \emph{specs} with properties of \emph{data_set_df} for the plot function.
+#' @seealso \url{https://adaemmerp.github.io/lpirfs/README_docs.html}
 #'
-#'\item{irf_lin_mean}{A three 3D \link{array}() containing all impulse responses for all endogenous variables.
+#' @return A list containing:
+#'
+#'
+#'\item{irf_lin_mean}{A three 3D \link{array} containing all impulse responses for all endogenous variables.
 #'                    The last dimension denotes the shock variable. The row in each matrix
-#'                    gives the responses of the \emph{ith} variable, ordered as in data_set_df. The columns denote the horizon.
+#'                    gives the responses of the \emph{ith} variable, ordered as in endog_data. The columns denote the horizons.
 #'                    For example, if \emph{results_lin} contains the list with results, results_lin$irf_lin_mean[, , 1] returns a KXH matrix,
-#'                    where K is the number of variables and H the number of horizons. '1' is the variable shock variable, corresponding to the
-#'                    variable in the first column of \emph{data_set_df}.}
+#'                    where K is the number of variables and H the number of horizons. '1' is the shock variable, corresponding to the
+#'                   first variable in \emph{endog_data}.}
 #'
-#'\item{irf_lin_low}{A three 3D \link{array}() containing all lower confidence bands of the responses,
+#'\item{irf_lin_low}{A three 3D \link{array} containing all lower confidence bands of the responses,
 #'                    based on robust standard errors by Newey and West (1987). Properties are equal to irf_lin_mean.}
 #'
-#'\item{irf_lin_up}{A three 3D \link{array}() containing all upper confidence bands of the responses,
+#'\item{irf_lin_up}{A three 3D \link{array} containing all upper confidence bands of the responses,
 #'                    based on robust standard errors by Newey and West (1987). Properties are equal to \emph{irf_lin_mean}.}
 #'
-#'\item{specs}{A list with properties of \emph{data_set_df} for the plot function.}
+#'\item{specs}{A list with properties of \emph{endog_data} for the plot function. It also contains
+#'             lagged data (y_lin and x_lin) used for the irf estimations.}
 #'
 #' @export
 #' @references
 #' Akaike, H. (1974). "A new look at the statistical model identification", \emph{IEEE Transactions on Automatic Control}, 19 (6): 716–723.
 #'
-#' Hurvich, C. M., and Tsai, C.-L. (1993) “A Corrected Akaike Information Criterion for
-#' Vector Autoregressive Model Selection.” \emph{Journal of Time Series Analysis}, 14(3):
-#' 271–79.
+#' Hurvich, C. M., and Tsai, C.-L. (1989), "Regression and time series model selection in small samples",
+#' \emph{Biometrika}, 76(2): 297–307
 #'
 #' Jordà, Ò. (2005). "Estimation and Inference of Impulse Responses by Local Projections."
 #' \emph{American Economic Review}, 95 (1): 161-182.
 #'
-#' Newey W.K., and West K.D. (1987). “A Simple, Positive-Definite, Heteroskedasticity and
+#' Newey, W.K., and West, K.D. (1987). “A Simple, Positive-Definite, Heteroskedasticity and
 #' Autocorrelation Consistent Covariance Matrix.” \emph{Econometrica}, 55: 703–708.
 #'
 #' Schwarz, Gideon E. (1978). "Estimating the dimension of a model", \emph{Annals of Statistics}, 6 (2): 461–464.
@@ -51,56 +59,138 @@
 #' @import foreach
 #' @examples
 #'\donttest{
+#'
+#'           ## Example without exogenous variables
+#'
 #'# Load package
 #'   library(lpirfs)
 #'
-#'# Load data
-#'   data_set_df <- interest_rules_var_data
+#'# Load (endogenous) data
+#'   endog_data <- interest_rules_var_data
 #'
 #'# Estimate linear model
-#'   results_lin <- lp_lin(data_set_df, lags_lin       = 4,
-#'                                      lags_criterion = NaN,
-#'                                      max_lags       = NaN,
-#'                                      trend          = 0L,
-#'                                      shock_type     = 1L,
-#'                                      confint        = 1.96,
-#'                                      hor            = 12)
+#'   results_lin <- lp_lin(endog_data,
+#'                              lags_endog_lin = 4,
+#'                              exog_data      = NULL,
+#'                              lags_exog      = NULL,
+#'                              lags_criterion = NaN,
+#'                              max_lags       = NaN,
+#'                              trend          = 0,
+#'                              shock_type     = 1,
+#'                              confint        = 1.96,
+#'                              hor            = 12,
+#'                              contemp_data   = NULL,
+#'                              num_cores      = NULL)
 #'
 #'# Make plots
-#'   linear_plots <- plot_lin_irfs(results_lin)
+#'  linear_plots <- plot_lin(results_lin)
 #'
 #'# Show single plots
+#'  # * The first element of 'linear_plots' shows the response of the first
+#'  #   variable (GDP_gap) to a shock in the first variable (GDP_gap).
+#'  # * The second element of 'linear_plots' shows the response of the first
+#'  #   variable (GDP_gap) to a shock in the second variable (inflation).
+#'  # * ...
+#'
 #'   linear_plots[[1]]
 #'   linear_plots[[2]]
 #'
+#'# Show all plots by using 'ggpubr' and 'gridExtra'
+#'# The package does not depend on those packages so they have to be installed
+#'  library(ggpubr)
+#'  library(gridExtra)
+#'
+#'# Compare with Figure 5 in Jordà (2005)
+#'  lin_plots_all <- sapply(linear_plots, ggplotGrob)
+#'  marrangeGrob(lin_plots_all, nrow = ncol(endog_data), ncol = ncol(endog_data), top = NULL)
+#'
+#'
+#'                       ## Example with exogenous variables ##
+#'
+#'# Load (endogenous) data
+#'  endog_data <- interest_rules_var_data
+#'
+#'# Create exogenous data and data with contemporaneous impact (for illustration purposes only)
+#'  exog_data    <- endog_data$GDP_gap*endog_data$Infl*endog_data$FF + rnorm(dim(endog_data)[1])
+#'  contemp_data <- endog_data$GDP_gap*endog_data$Infl*endog_data$FF + rnorm(dim(endog_data)[1])
+#'
+#'# Exogenous data has to be a data.frame
+#'  exog_data    <- data.frame(xx = exog_data )
+#'  contemp_data <- data.frame(cc =  contemp_data)
+#'
+#'# Estimate linear model
+#'   results_lin <- lp_lin(endog_data,
+#'                                lags_endog_lin = 4,
+#'                                lags_criterion = NaN,
+#'                                max_lags       = NaN,
+#'                                trend          = 0,
+#'                                shock_type     = 1,
+#'                                confint        = 1.96,
+#'                                hor            = 12,
+#'                                exog_data      = exog_data,
+#'                                lags_exog      = 4,
+#'                                contemp_data   = contemp_data,
+#'                                num_cores      = NULL)
+#'
+#'# Make plots
+#'  linear_plots <- plot_lin(results_lin)
+#'
 #'# Show all plots
-#'   library(ggpubr)
-#'   library(gridExtra)
+#'  library(ggpubr)
+#'  library(gridExtra)
+#'
+#'  lin_plots_all <- sapply(linear_plots, ggplotGrob)
+#'  marrangeGrob(lin_plots_all, nrow = ncol(endog_data), ncol = ncol(endog_data), top = NULL)
 #'
-#'   lin_plots_all <- sapply(linear_plots, ggplotGrob)
-#'   marrangeGrob(lin_plots_all, nrow = ncol(data_set_df), ncol = ncol(data_set_df), top = NULL)
 #'
 #'  }
-lp_lin <- function(data_set_df, lags_lin = NULL, lags_criterion = NULL, max_lags = NULL,
-                                trend    = NULL, shock_type    = NULL,  confint  = NULL, hor = NULL){
+lp_lin <- function(endog_data,
+                        lags_endog_lin = NULL,
+                        lags_criterion = NULL,
+                        max_lags       = NULL,
+                        trend          = NULL,
+                        shock_type     = NULL,
+                        confint        = NULL,
+                        hor            = NULL,
+                        exog_data      = NULL,
+                        lags_exog      = NULL,
+                        contemp_data   = NULL,
+                        num_cores      = NULL){
 
   # Create list to store inputs
     specs <- list()
 
   # Specify inputs
-    specs$lags_lin       <- lags_lin
+    specs$lags_endog_lin <- lags_endog_lin
     specs$lags_criterion <- lags_criterion
     specs$max_lags       <- max_lags
     specs$trend          <- trend
     specs$shock_type     <- shock_type
     specs$confint        <- confint
     specs$hor            <- hor
+    specs$exog_data      <- exog_data
+    specs$lags_exog      <- lags_exog
+
+    # Add 'contempranoeus' as NULL for data construction
+    specs$contemp_data   <- contemp_data
+    # Set model type for lag construction
+    specs$model_type     <- 0
 
   # Check whether data is a data.frame()
-  if(!(is.data.frame(data_set_df))){
+  if(!(is.data.frame(endog_data))){
     stop('The data has to be a data.frame().')
   }
 
+  # Give message when no linear model is provided
+    if(is.null(exog_data)){
+      message('You estimate the model without exogenous data.')
+    }
+
+    # Give message when no contemporaneous data is provided
+    if(is.null(contemp_data)){
+      message('You estimate the model without exogenous data with contemporaneous impact.')
+    }
+
   # Check whether 'trend' is given
   if(is.null(specs$trend)){
     stop('Please specify whether and which type of trend to include.')
@@ -131,14 +221,14 @@ lp_lin <- function(data_set_df, lags_lin = NULL, lags_criterion = NULL, max_lags
 
   # Check whether lags criterion and maximum number of lags are given
   if((is.character(specs$lags_criterion)) &
-      (!is.na(specs$lags_lin))){
+      (!is.na(specs$lags_endog_lin))){
      stop('You can not provide a lag criterion (AICc, AIC or BIC) and a fixed number of lags.')
     }
 
 
   # Check whether no lag length criterion and number of lags are given
   if((is.na(specs$lags_criterion)) &
-      (is.na(specs$lags_lin))){
+      (is.na(specs$lags_endog_lin))){
     stop('You have to at least provide a lag criterion (AICc, AIC or BIC) or a fixed number of lags.')
   }
 
@@ -156,8 +246,8 @@ lp_lin <- function(data_set_df, lags_lin = NULL, lags_criterion = NULL, max_lags
   }
 
   # Check whether lags for linear model are integers
-  if(is.numeric(specs$lags_lin) & !is.nan(specs$lags_lin)){
-    if(!(specs$lags_lin %% 1 == 0)  | specs$lags_lin < 0){
+  if(is.numeric(specs$lags_endog_lin) & !is.nan(specs$lags_endog_lin)){
+    if(!(specs$lags_endog_lin %% 1 == 0)  | specs$lags_endog_lin < 0){
       stop('The numbers of lags have to be a positive integer.')
     }
   } else {}
@@ -183,20 +273,36 @@ lp_lin <- function(data_set_df, lags_lin = NULL, lags_criterion = NULL, max_lags
       stop('The maximum number of lags is only used if you provide a lag length criterion.')
     }
 
+  # Check whether exogenous data is a data.frame
+    if(!is.null(specs$exog_data) & !is.data.frame(specs$exog_data)){
+      stop('Exogenous data has to be a data.frame.')
+    }
+
+  # Check whether lag length for exogenous data is given
+    if(!is.null(specs$exog_data) & is.null(specs$lags_exog)){
+      stop('Please provide a lag length for the exogenous data.')
+    }
+
 
   # Safe data frame specifications in 'specs for functions
    specs$starts         <- 1                        # Sample Start
-   specs$ends           <- dim(data_set_df)[1]      # Sample end
-   specs$columns        <- names(data_set_df)       # Name endogenous variables
-   specs$endog          <- ncol(data_set_df)        # Set the number of endogenous variables
+   specs$ends           <- dim(endog_data)[1]      # Sample end
+   specs$column_names   <- names(endog_data)       # Name endogenous variables
+   specs$endog          <- ncol(endog_data)        # Set the number of endogenous variables
 
- # Construct (lagged) data
-  data_lin <- create_lin_data(specs, data_set_df)
+ # Construct (lagged) endogenous data
+  data_lin <- create_lin_data(specs, endog_data)
   y_lin    <- data_lin[[1]]
   x_lin    <- data_lin[[2]]
 
+
+  # Save endogenous and lagged exogenous data in specs
+  specs$y_lin        <- y_lin
+  specs$x_lin        <- x_lin
+
+
  # Construct shock matrix
-  d <- get_mat_chol(y_lin, x_lin, data_set_df, specs)
+  d <- get_mat_chol(y_lin, x_lin, endog_data, specs)
 
  # Matrices to store OLS parameters
   b1            <- matrix(NaN, specs$endog, specs$endog)
@@ -213,9 +319,12 @@ lp_lin <- function(data_set_df, lags_lin = NULL, lags_criterion = NULL, max_lags
   irf_lin_low   <-  irf_lin_mean
   irf_lin_up    <-  irf_lin_mean
 
- # Make cluster
-  numb_cores     <- min(specs$endog, parallel::detectCores() - 1)
-  cl             <- parallel::makeCluster(numb_cores)
+  # Make cluster
+  if(is.null(num_cores)){
+    num_cores     <- min(specs$endog, parallel::detectCores() - 1)
+  }
+
+  cl             <- parallel::makeCluster(num_cores)
   doParallel::registerDoParallel(cl)
 
  # Decide whether lag lengths are given or have to be estimated
@@ -226,7 +335,7 @@ lp_lin <- function(data_set_df, lags_lin = NULL, lags_criterion = NULL, max_lags
                       .packages = 'lpirfs')  %dopar%{ # Accounts for the shocks
 
    for (h in 1:(specs$hor)){   # Accounts for the horizons
-I
+
     # Create data
      yy  <-   y_lin[h : dim(y_lin)[1], ]
      xx  <-   x_lin[1 : (dim(x_lin)[1] - h + 1), ]
@@ -288,18 +397,19 @@ I
       for (k in 1:specs$endog){ # Accounts for endogenous reactions
 
         # Find optimal lags
+         n_obs         <- nrow(y_lin[[1]]) - h + 1 # Number of observations for model with lag one
          val_criterion <- lpirfs::get_vals_lagcrit(y_lin, x_lin, lag_crit, h, k,
-                                                 specs$max_lags)
+                                                   specs$max_lags, n_obs)
 
         # Set optimal lag length
-         lag_choice  <- which.min(val_criterion)
+         lag_choice    <- which.min(val_criterion)
 
         # Extract matrices based on optimal lag length
-         yy <- y_lin[[lag_choice]][, k]
-         yy <- yy[h: length(yy)]
+         yy            <- y_lin[[lag_choice]][, k]
+         yy            <- yy[h: length(yy)]
 
-         xx <- x_lin[[lag_choice]]
-         xx <- xx[1:(dim(xx)[1] - h + 1),]
+         xx            <- x_lin[[lag_choice]]
+         xx            <- xx[1:(dim(xx)[1] - h + 1),]
 
         # Estimate coefficients and newey west std.err
          nw_results   <- lpirfs::newey_west(yy, xx, h)
@@ -345,6 +455,6 @@ I
   parallel::stopCluster(cl)
 
   list(irf_lin_mean = irf_lin_mean, irf_lin_low = irf_lin_low,
-       irf_lin_up   = irf_lin_up, spes = specs)
+       irf_lin_up   = irf_lin_up, specs = specs)
 
 }
